#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"


#define VENDOR_RQ_GET_SWITCH1 0
#define VENDOR_RQ_GET_SWITCH2 1
#define VENDOR_RQ_GET_SOUND1  2
#define VENDOR_RQ_GET_SOUND2  3
#define VENDOR_RQ_SET_LED     4





/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 

void out_b(uint8_t pin, uint8_t val)
{
    // write logic level to port-D's pin
    DDRB |= 1<<pin;
    if (val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}

uint16_t read_adc(uint8_t channel)
{
    ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    ADCSRA = 0b11000110;
    //         ||   -+-
    //         ||    |
    //         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    //         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    //         |
    //         +-> Enable ADC (บิต ADEN)

    while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        ;
    return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
}

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[2];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_b(led_no, led_val);
        return 0;
    }


    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH1)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH2)
    {
        dataBuffer[0] = !in_c(2);
        usbMsgPtr = dataBuffer;
        return 1;
    }
  else if (rq->bRequest == VENDOR_RQ_GET_SOUND1)
    {
		uint16_t sound = read_adc(5);     
		dataBuffer[0] = sound % 256;
		dataBuffer[1] = sound / 256;
        usbMsgPtr = dataBuffer;
        return 2;
  }
  else if (rq->bRequest == VENDOR_RQ_GET_SOUND2)
    {
		uint16_t sound = read_adc(4);     
		dataBuffer[0] = sound % 256;
		dataBuffer[1] = sound / 256;
        usbMsgPtr = dataBuffer;
        return 2;
  }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */
#define VENDOR_RQ_GET_SWITCH 1
int main(void)
{
	//uint16_t sound1,sound2,n=10;

    DDRC  = 0b00000000;
    PORTC = 0b00001100;
	DDRB = 0b11111111;
	PORTB = 0b00000000;
	


    usbInit();
    usbDeviceDisconnect();  /* enforce re-enumeration, do this while interrupts are disabled! */
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                /* main event loop */
        usbPoll();
    }
    return 0;
}

/* ------------------------------------------------------------------------- */
